#ifndef Event_h
#define Event_h

#include <vector>
#include "HecCell.h"
#include "Geometry.h"
#include "EmecCell.h"

/*!  - Class Event
  * - Author: Ian Gable (igable@uvic.ca)
  * - Date:  05/10/2002
  * - Description: Event stores a list of HecCells in a vector. Each HecCell can be retrieved.
  */
class Event{

 public:
  /** The default contructor
    */
  Event();

  //========HecPortion=====================================

  /** Add adcValue to hecCell with adcNumber
    * @param cell a static HecCell Class
    */
  bool addHecCell(HecCell cell);

  /** Return a pointer to hec Cell with adcNumber
    * @param   acdNumber The acd Number of the cell you wish to return
    */
  HecCell* hecCell(int adcNumber);

   //! clear the content of the cells so that  new info can be loaded
  void clearHecCells();

  //! return the number of cells
  int numHecCells();

  // ! return a constant pointer to the m_hecCells vector
  const vector<HecCell>* hecCellsPointer();

  //! get event number
  int hecEventNumber() { return m_hecEventNumber; };

  //! set event number
  void hecEventNumber(int eventNum) {m_hecEventNumber = eventNum;};

  /** Set the hecTrigger. Note that the parameter MUST be an array of
    * 5 integers.
    * @param hecTriggerArray a pointer to the hecTriggerArray
    */
  void setHecTrigger(unsigned int hecTriggerArray[5]);

  //! Return a pointer to the HecTriggerVector
  vector<int>* getHecTriggerVector(){ return &m_hecTriggerVector;};

  //! Returns true when particle has contancted both muon detectors
  bool isHecMuon() { return m_isHecMuon;}

  //! Returns true when event is a Random Trigger
  bool isHecRandom() { return m_isHecRandom;}

  /** Get the trigger bit from 0 to 4
    * (physics,electron,muon,pion,random)
    * @param bit The trigger bit 0-4 will cause exit(1) if accessed outside
    */
  int getHecTriggerBit(int bit);


//=========EmecPortion=======================

  /** Add adcValue to EmecCell with adcNumber
    * @param cell a static EmecCell Class
    */
  bool addEmecCell(EmecCell cell);

  /** Return a pointer to EmecCell with adcNumber
    * @param   acdNumber The acd Number of the cell you wish to return
    */
  EmecCell* emecCell(int adcNumber);

   //! clear the content of the cells so that  new info can be loaded
  void clearEmecCells();

  //! return the number of cells
  int numEmecCells();

  // ! return a constant pointer to the m_emecCells vector
  const vector<EmecCell>* emecCellsPointer();

  //! get event number
  int emecEventNumber() { return m_emecEventNumber; };

  //! set event number
  void emecEventNumber(int eventNum) {m_emecEventNumber = eventNum;};

  /** Set the emecTrigger. Note that the parameter MUST be an array of
    * 5 integers.
    * @param emecTriggerArray a pointer to the emecTriggerArray
    */
  void setEmecTrigger(unsigned int emecTriggerArray[5]);

  //! Return a pointer to the EmecTriggerVector
  vector<int>* getEmecTriggerVector(){ return &m_emecTriggerVector;};

  //! Returns true when particle has contancted both muon detectors
  bool isEmecMuon() { return m_isEmecMuon;}

  //! Returns true when event is a Random Trigger
  bool isEmecRandom() { return m_isEmecRandom;}

  /** Get the trigger bit from 0 to 4
    * (physics,electron,muon,pion,random)
    * @param bit The trigger bit 0-4 will cause exit(1) if accessed outside
    */
  int getEmecTriggerBit(int bit);

  //maybe include method to get individual elements of the Trigger Vector


  //! Get the beam chamber X Slope
  float beamChamberSlopeX() {return m_bcSlopeX;};

  //! Get the BeamChamber YSlope
  float beamChamberSlopeY() {return m_bcSlopeY;};

  //! Get the beam chamber X position at z = 0
  float beamChamberXatZ0() {return m_bcXatZ0; };

  //! Get the beam chamber y position at z = 0
  float beamChamberYatZ0(){ return m_bcYatZ0; };

  //!
  float beamChamberCovXatZ0() { return m_bcCovXatZ0;};

  //!
  float beamChamberCovXatSlopeX(){ return  m_bcCovXatSlopeX;};

  //!
  float beamChamberCovXatZ0SlopeX(){ return m_bcCovXatZ0SlopeX;};

  //!
  float beamChamberCovYatZ0() { return m_bcCovXatZ0;};

  //!
  float beamChamberCovYatSlopeX(){ return  m_bcCovXatSlopeX;};

  //!
  float beamChamberCovYatZ0SlopeX(){ return m_bcCovXatZ0SlopeX;};

  void setBeamChamberInfo(float slope[2], float pos0[2], float xcov[3], float ycov[3]);

  /** Return a STL vector of floats that are are the parameters for the
    * beam chamber info. The order of the parameters is a0, a1, b0, b1.
    */
  vector<float> beamChamberLine();

 private:
  vector<HecCell> m_hecCellVector; //! contains an array of cells
  void processHecTrigger();

  int m_hecEventNumber; //! the event number
  vector<int> m_hecTriggerVector; //! vector of the trigger information
  bool m_isHecMuon;
  bool m_isHecRandom;

  vector<EmecCell> m_emecCellVector; //! contains an array of cells
  void processEmecTrigger();

  int m_emecEventNumber; //! the event number
  vector<int> m_emecTriggerVector; //! vector of the trigger information
  bool m_isEmecMuon;
  bool m_isEmecRandom;

  // beam chamber info
  float m_bcSlopeX;
  float m_bcSlopeY;
  float m_bcXatZ0;
  float m_bcYatZ0;

  float m_bcCovXatZ0;
  float m_bcCovXatSlopeX;
  float m_bcCovXatZ0SlopeX;

  float m_bcCovYatZ0;
  float m_bcCovYatSlopeY;
  float m_bcCovYatZ0SlopeY;



};
#endif
