#ifndef TBID_h
#define TBID_h

//#include <function>

/*! - Class:    TBID
      - Author:   Ian Gable (igable@uvic.ca)
                     Naoko Kanaya (nkanaya@uvic.ca)
      - Date:      07/07/2003
      - Description: TBID is a test beam cell ID class that provides a unique integer for each cell in the combined
                Beam Test of 2002. This class can be used in two different ways. In the first way the class holds as a
                member, the cells ID, which is created when the object is constructed or alternatively it can be used as
                a tool for converting between detector ID and the various integer coordinates (idet,ieta,iphi,iz).  
                In the second case the constructor which takes no parameters is used.
  */


class TBID{

 public:

  /** Constructor to be used when you what the class to store its cell ID and then later access the
    * various coordinates (idet,ieta,iphi,iz) without giving a detector id.
    *@param idet  Detector id: hec = 2, emec = 1
    *@param ieta hec = [ 1,14], emec = [ 0,67]
    *@param iphi hec=[ 1, 6], emec = [ 0,31]
    *@param iz hec = [ 1, 3], emec = [ 0, 3]
    */
  TBID(int idet, int ieta, int iphi, int iz);

  /** Constructor to be used when the class used as simply a conversion tool.
    */
  TBID(){};

  //! Return the stored cell id
  int id();

  /** Given cell integer coordinates return an ID.
    *@param idet  Detector id: hec = 2, emec = 1
    *@param ieta hec = [ 1,14], emec = [ 0,67]
    *@param iphi hec=[ 1, 6], emec = [ 0,31]
    *@param iz hec = [ 1, 3], emec = [ 0, 3]
    */
  int id(int idet, int ieta, int iphi, int iz);

  //! Return the stored ieta coordinate
  int ieta();

  /** Given a cell ID return coresponding ieta coord
    *@param ID the ID of the cell to retrieve the ieta coordinate
    */
  int ieta(int ID);

  //! Return the stored iphi coordinate
  int iphi();

  /** Given a cell ID return coresponding iphi coord
    *@param ID the ID of the cell to retrieve the iphi coordinate
    */
  int iphi(int ID);

  //! Return the stored iz coordinate
  int iz();

  /** Given a cell ID return coresponding iz coord
    *@param ID the ID of the cell to retrieve the iz coordinate
    */
  int  iz(int ID);

  //! Return the stored detector number (hec = 2, emec =1)
  int idet();

  /** Given a cell ID return coresponding detector number (hec = 2, emec =1)
    *@param ID the ID of the cell to retrieve the iz coordinate
    */
  int idet(int ID);

 //! The less than operator
 friend bool operator<(const TBID &,const TBID &);

  //! The equal operator
  friend bool operator==(const TBID &,const TBID &);

 private:

  enum { BIT_PHI=7, BIT_Z=14, BIT_DET=21 };

  int m_id;


};

inline bool operator<(const TBID &id1,const TBID &id2){
  return id1.m_id < id2.m_id;
}

inline bool operator==(const TBID &id1,const TBID &id2){
  return id1.m_id == id2.m_id;
}


#endif
