#include <iostream>
#include <unistd.h> //included for the sleep function
#include <getopt.h>
#include "TROOT.h"

#include "Event.h"
#include "HecEvent.h"
#include "HecCell.h"
#include "EmecCell.h"
#include "HecRunHeader2002v1.h"
#include "HecEventNTup2002v1.h"
#include "GlobalHeader2002v0.h"
#include "EmecRunHeader2002v0.h"
#include "EmecEventNTup2002v1.h"
#include "SlowControl2002v0.h"
#include "Headers.h"
#include "Geometry.h"
#include "TApplication.h"
#include "App.h"
#include "TBrowser.h"




#include "UserAlg.h"

using std::cerr;

/*! Class:    No Class. This file contains the main() function.
  * Author:   Ian Gable (igable@uvic.ca)
  * Date:      05/10/2002
  * Description:  main() parsses the command line arguments
  */

// The main control loop for the program.
// This method handles creating the necessary HecEvent, Geometry and UserAlg
// objects. It then passes control to the user at the apropriate time.
void run(string hecFileName, string emecFileName, string hecGeoFileName,string emecGeoFileName,string globalGeoFileName){


  //create all the objects which are found in a standard combined NTuple
  HecEventNTup2002v1* hecNTuple = new HecEventNTup2002v1(0,hecFileName);
  HecRunHeader2002v1* hecHeader = new HecRunHeader2002v1(0,hecFileName);
  EmecEventNTup2002v1* emecNTuple = new EmecEventNTup2002v1(0,emecFileName);
  EmecRunHeader2002v0* emecHeader = new EmecRunHeader2002v0(0,emecFileName);

  SlowControl2002v0* slowControl =  new SlowControl2002v0(0,hecFileName);

  GlobalHeader2002v0* globalCheck = new GlobalHeader2002v0(0,emecFileName);
  GlobalHeader2002v0* global = new GlobalHeader2002v0(0,hecFileName);

  //add all the header objects to the headers wrapper class
  Headers* headers = new Headers(hecHeader, emecHeader,slowControl,global);

  
  //here we load the information from the header NTuple
  hecHeader->LoadTree(0);
  hecHeader->fChain->GetEntry(0);

  emecHeader->LoadTree(0);
  emecHeader->fChain->GetEntry(0);

  global->LoadTree(0);
  global->getChain()->GetEntry(0);

  globalCheck->LoadTree(0);
  globalCheck->getChain()->GetEntry(0);

  slowControl->LoadTree(0);
  slowControl->fChain->GetEntry(0);

  cout << "Hec Run Number: " << global->getRunNo() << endl;
  cout << "Emec Run Number: " << globalCheck->getRunNo() <<endl;
  if(global->getRunNo() != globalCheck->getRunNo()){
    cout << "\n+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+\n"
      << "WARNING!!!!!!! : EMEC and HEC Run Numbers are not the same\n"
      << "Do you have the right files?\n\n";
  }

  const int hecFirstFebno = headers->hec()->getFirstCh();
  const int hecLastFebno = headers->hec()->getLastCh();
  const int emecFirstFebno = headers->emec()->getFirstCh();
  const int emecLastFebno = headers->emec()->getLastCh();
  cout << "Run header: hec febno from " << hecFirstFebno << " to " << hecLastFebno  << endl;
  cout << "Run header: emec febno from " << emecFirstFebno << " to " << emecLastFebno << endl;
  int numEvents = hecNTuple->numEvents();

  cout << "Number of Events: " << numEvents << endl;

  Geometry* hecGeo = new Geometry(hecGeoFileName,emecGeoFileName, globalGeoFileName);

  Event* event = new Event();

  cout << "Creating UserAlg ...\n";
  UserAlg* userAlg = new UserAlg(event,hecGeo,headers);

  cout << "Excuting UserAlg::initialize() ...\n";
  userAlg->initialize(); // User controled initialization

  cout << "Beginnning event  Loop... \n";
  //for now I'm going to make the presuption that we have the same number of events

  for(int indexE = 0; indexE < numEvents; indexE++){
    if (indexE%1000 == 0) cout << "Processing event number " << indexE << endl;

    hecNTuple->LoadTree(indexE);

    hecNTuple->fChain->GetEntry(indexE); // very important apparently

    event->hecEventNumber(hecNTuple->evtno );
    event->setHecTrigger(hecNTuple->trig); // this is a pointer to an array;
    event->setBeamChamberInfo(hecNTuple->slop,hecNTuple->pos0,hecNTuple->xcov,hecNTuple->ycov);

    for(int febno = hecFirstFebno;  febno <= hecLastFebno; febno++){
      HecCell cell(febno,hecNTuple->signal[febno -1]); // memory inefficient but may be unimportant

      event->addHecCell(cell);

      if(hecNTuple->signal[febno -1] > 100){
        //cout << "Index for value " << hecNTuple->hec_signal[febno-1]  << " is: " << febno -1 << endl;
      }
    }

    emecNTuple->LoadTree(indexE);
    emecNTuple->fChain->GetEntry(indexE); // very important apparently

    event->emecEventNumber(emecNTuple->evtno );
    event->setEmecTrigger(emecNTuple->trig); // this is a pointer to an array;

    //note that for the Emec the channels are numbered 0 to x
    for(int febno = emecFirstFebno; febno <= emecLastFebno; febno++){
      EmecCell ecell(febno,emecNTuple->signal[febno]); // memory inefficient but may be unimportant

      event->addEmecCell(ecell);

      if((emecNTuple->signal[febno] > 100) || (emecNTuple->signal[febno] < -100)){
        //cout << "Index for value " << emecNTuple->signal[febno]  << " is: " << febno  << endl;
      }
    }

    if(event->hecEventNumber() != event->emecEventNumber()){
      cout << "\n+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+\n"
      << "WARNING!!!!!!! : HEC and Emec event numbers mismatched\n"
      << "Posible HEC EMEC out of phase problem?\n\n";
    }

    userAlg->execute(); // give control to user
    event->clearHecCells(); // clear the cell so that new values can be loaded
    event->clearEmecCells();
  }
  cout << "Executing UserAlg::finalize() ...\n";
  userAlg->finalize(); //let the user do some cleanup

  delete hecNTuple;
  cout << "Deleted hecNTuple" << endl;
  delete hecHeader;
  cout << "Deleted hecHeader" << endl;
  delete userAlg;
  cout << "Deleted userAlg" <<endl;
  delete event;
  cout << "Deleted Event" << endl;
  delete hecGeo;
  cout << "Deleted hecGeo" << endl;
}


int  main(int argc, char * argv[]){

  TApplication myapp("myapp",0,0); //this will make the X11 libs load and allow you
                                                  // to draw objects. Fix for 3.05

  gROOT->SetApplication(&myapp);
  //myapp.Run();


  cout << "Begining Execution... \n";
  gROOT->GetListOfClasses();


  string commandArgs(*argv);
  string pathToExecutable("");
  string::size_type end =commandArgs.find_first_of(" ");
  string firstThing = commandArgs.substr(0,end);
  string::size_type lastSlash = firstThing.find_last_of("/");
  if(lastSlash < firstThing.size()){
    pathToExecutable = firstThing.substr(0,lastSlash+1);
    cout << "Path: " <<  pathToExecutable <<endl;
  }

  // parse the command line arguments
  // do not modify this code unless you're very brave!
  string hecFileName;
  string emecFileName;
  string hecGeoFileName("");
  string emecGeoFileName("");
  string globalGeoFileName("");
  
  vector<string> hecFileVector;
  vector<string> emecFileVector;
     
  bool hecFile= false;
  bool isEmecGeoFile= false;
  bool isHecGeoFile= false;
  bool isEmecFile= false;
  bool isHecFile =false;

  if(argc > 1){
    for(int i = 0; i <argc-1;i++){
    
      char* pchar = argv[i+1];
 

      switch( pchar[0] ){
      case '-': {
        switch( pchar[1] ){
	case 'h':
	  isHecFile = true;
	  isHecGeoFile = false;
	  isEmecFile = false;
	  isEmecGeoFile =false;
	  break;
	case 'e':
	  isEmecFile = true;
	  isHecGeoFile =false;
	  isHecFile = false;
	  isEmecGeoFile =false;
	  break;
	case 'a':
	  isHecGeoFile = true;
	  isHecFile =false;
	  isEmecFile = false;
	  isEmecGeoFile = false;
	  break;
	case 'b':
	  isEmecGeoFile= true;
	  isHecFile= false;
	  isEmecFile=false;
	  isHecGeoFile=false;
	  break;
	default:
	  cout << "Unrecognized option -" << pchar[1] << endl;
	  exit(1);
	  break;
	}
	break;
      }
      default: {
	//handle the file names
        if(isHecFile){
	  hecFileName = pchar;
	  hecFileVector.push_back(hecFileName);
        } else if(isHecGeoFile){
	  hecGeoFileName = pchar;
	  isHecGeoFile =false;
	} else if(isEmecGeoFile){
	  emecGeoFileName = pchar;
	  isHecGeoFile = false; 
        } else if(isEmecFile){
	  emecFileName = pchar;
	  emecFileVector.push_back(emecFileName);
	} else {
	  // handle more cases
	}
	break;
      }
      }

    }
  } else {
    cout << "Must specify both a Hec File and a Emec File geometry file:\n"
	 << "Example: tbrootana -e emecFileName -h hecFileName -g GeometryFileName"
	 << endl;
    exit(1);
  }
    

  //check to see if we are using default geometry
  if(hecGeoFileName == ""){
    hecGeoFileName = pathToExecutable + "../geo/hec_geo_030629.dat";
    cout << "Using default hec geometry" << endl;
  }
  if(emecGeoFileName == ""){
    emecGeoFileName = pathToExecutable + "../geo/emec_geo_030610.dat";
    cout << "Using default emec geometry" << endl;
  }
  if(globalGeoFileName == ""){
    globalGeoFileName = pathToExecutable + "../geo/global_geo_030617.dat";
    cout << "Using default global geometry" << endl;
  }

  cout << "Hec Geometry File Name: " << hecGeoFileName <<endl;
  cout << "Emec Geometry File Name: " <<emecGeoFileName <<endl;
  cout << "Global Geometry FileName: " << globalGeoFileName << endl;
  cout << "Hec File Name     : " << hecFileName << endl;
  cout << "Emec File Name    : " << emecFileName <<endl;
  vector<string>::iterator itr;

  if(hecFileVector.size() != emecFileVector.size()){
    cout << "ERRROR: Not the same number of Hec and Emec Files!!\n";
    exit(1);
  }

  if(hecFileVector.size() > 1){
    cout << "*********************************\n"
         << "** TBRootAna Processing a      **\n"
         << "** a batch job                 **\n"
         << "*********************************\n";

    cout << "Files That Will be Processed\n";
    for(itr = hecFileVector.begin(); itr != hecFileVector.end(); itr++){
      cout << *itr << endl;
    }
  }
  sleep(3);// let people look at the list for three seconds

  for(int i = 0; i <hecFileVector.size();i++){
    run(hecFileVector[i], emecFileVector[i],hecGeoFileName,emecGeoFileName,globalGeoFileName);
  }
  App mainWindow(gClient->GetRoot(), 400, 220);
 //TBrowser*  b = new TBrowser();
  //myapp.Run();

  myapp.Run();
  return 0;

}

